'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var getSha = require('./get-sha-86c6b62d.js');
require('fs');
require('path');
require('child_process');
require('stream');
require('assert');
require('events');
require('util');

/** Helper function to execute {@link https://git-scm.com/docs/git-notes#_subcommands git-notes subcommands}
 * @param sha SHA of commit, blob, or tree
 * @param options accept either value of `--ref` or `--ref` + {@link https://github.com/sindresorhus/execa#options execa Options}
 * @return list of function correspond to git-notes subcommands which can be called
 */
function gitNotes(sha, options) {
  const { ref, ...execaOpts } = options;
  return {
    add: notes =>
      getSha.execa_4(
        'git',
        ['notes', ...(ref ? ['--ref', ref] : []), 'add', sha, '-m', notes],
        execaOpts
      ),
    overwriteWith: text =>
      getSha.execa_4(
        'git',
        [
          'notes',
          ...(ref ? ['--ref', ref] : []),
          ...(getSha.isHash(text)
            ? ['copy', text, sha]
            : ['add', sha, '-m', text]),
          '-f'
        ],
        execaOpts
      ),
    copyFrom: otherSHA =>
      getSha.execa_4(
        'git',
        ['notes', ...(ref ? ['--ref', ref] : []), 'copy', otherSHA, sha],
        execaOpts
      ),
    append: notes =>
      getSha.execa_4(
        'git',
        ['notes', ...(ref ? ['--ref', ref] : []), 'append', sha, '-m', notes],
        execaOpts
      ),
    remove: () =>
      getSha.execa_4(
        'git',
        ['notes', ...(ref ? ['--ref', ref] : []), 'remove', sha],
        execaOpts
      ),
    show: () =>
      getSha.execa_4(
        'git',
        ['notes', ...(ref ? ['--ref', ref] : []), 'show', sha],
        execaOpts
      ).stdout
  };
}

/** Class Handler to make 🆒 semantic API style 😎 */
class NotesHandler {
  constructor(opts, ops, notes) {
    this.opts = opts;
    this.ops = ops;
    this.notes = notes;
    /** Insert/Read notes at specific file
     * @param message must be string of the message
     * @returns chainable function **if possible**
     */
    this.atCommit = message => this.at(this.getCommitSHA(message));
    if (ops && !notes) throw new Error('Notes is missing for operation ' + ops);
  }
  /** Insert/Read notes at specific git hash-object
   * @param sha hash-object of commit, blob, or tree
   * @returns chainable function **if possible**
   */
  at(sha) {
    if (this.ops) return gitNotes(sha, this.opts)[this.ops](this.notes);
    else return gitNotes(sha, this.opts);
  }
  /** Insert/Read notes at specific file
   * @param file can be filename or relative filepath
   * @param commit can be commit-id (SHA-1) or commit-message
   * @returns chainable function **if possible**
   */
  atFile(file, commit) {
    if (getSha.isHash(commit)) return this.at(getSha.fromFile(file, commit));
    else return this.at(getSha.fromFile(file, this.getCommitSHA(commit)));
  }
  /** Insert/Read notes at specific folder
   * @param tree can be foldername or relativepath
   * @param commit can be commit-id (SHA-1) or commit-message
   * @returns chainable function **if possible**
   */
  atFolder(tree, commit) {
    if (getSha.isHash(commit)) return this.at(getSha.fromFolder(tree, commit));
    else return this.at(getSha.fromFolder(tree, this.getCommitSHA(commit)));
  }
  /** Insert/Read notes at specific submodule
   * @todo Need test 🙂
   *
   * @param submodule must be relativepath
   * @param commit can be commit-id (SHA-1) or commit-message
   * @returns chainable function **if possible**
   */
  atSubmodule(submodule, commit) {
    if (getSha.isHash(commit))
      return this.at(getSha.fromSubmodule(submodule, commit));
    else
      return this.at(
        getSha.fromSubmodule(submodule, this.getCommitSHA(commit))
      );
  }
  // same as `fromCommit` from `get-sha.ts` but with error handling
  getCommitSHA(message) {
    const commitId = getSha.fromCommit(message);
    if (Array.isArray(commitId))
      return getSha.becomePanic({
        operation: {
          name: 'commit-message',
          data: message
        },
        data: commitId,
        suggestion: `git log --all --grep='${message}' --oneline`
      });
    else return commitId;
  }
}

const isPlainObject = obj => !!obj && obj.constructor === Object;
/** Instantiate fluent API for git-notes 😎
 * @param text can be SHA or string notes
 * @param options can be --ref string or key-value (js object) of options
 * @return chainable function/object depend on `text`
 */
function index(text, options = {}) {
  if (isPlainObject(text)) options = text;
  else options = typeof options === 'string' ? { ref: options } : options;
  if (typeof text !== 'string') return new NotesHandler(options);
  else if (getSha.isHash(text)) return gitNotes(text, options);
  else
    return {
      // if isNotes
      add: new NotesHandler(options, 'add', text),
      overwrite: new NotesHandler(options, 'overwriteWith', text),
      copy: new NotesHandler(options, 'add', text),
      append: new NotesHandler(options, 'append', text)
    };
}

exports.notes = index;
